@@ -59,6 +59,8 @@ Project::Project (const File& f)
     setChangedFlag (false);
 
     projectRoot.addListener (this);
+
+    modificationTime = getFile().getLastModificationTime();
 }
 
 Project::~Project()
@@ -374,6 +376,20 @@ void Project::valueTreeChildRemoved (ValueTree&, ValueTree&, int)   { changed();
 void Project::valueTreeChildOrderChanged (ValueTree&, int, int)     { changed(); }
 void Project::valueTreeParentChanged (ValueTree&)                   {}
 
+//==============================================================================
+bool Project::hasProjectBeenModified()
+{
+    Time newModificationTime = getFile().getLastModificationTime();
+
+    if (newModificationTime != modificationTime)
+    {
+        modificationTime = newModificationTime;
+        return true;
+    }
+
+    return false;
+}
+
 //==============================================================================
 File Project::resolveFilename (String filename) const
 {
@@ -1215,15 +1231,13 @@ void Project::createExporterForCurrentPlatform()
 String Project::getFileTemplate (const String& templateName)
 {
     int dataSize;
-    const char* data = BinaryData::getNamedResource (templateName.toUTF8(), dataSize);
 
-    if (data == nullptr)
-    {
-        jassertfalse;
-        return String::empty;
-    }
+    if (const char* data = BinaryData::getNamedResource (templateName.toUTF8(), dataSize))
+        return String::fromUTF8 (data, dataSize);
+
+    jassertfalse;
+    return String();
 
-    return String::fromUTF8 (data, dataSize);
 }
 
 //==============================================================================
