@@ -33,16 +33,16 @@
 
   ID:               juce_audio_devices
   vendor:           juce
-  version:          4.2.3
+  version:          4.3.0
   name:             JUCE audio and MIDI I/O device classes
   description:      Classes to play and record from audio and MIDI I/O devices
   website:          http://www.juce.com/juce
   license:          GPL/Commercial
 
-  dependencies:     juce_audio_basics, juce_audio_formats, juce_events
-  OSXFrameworks:    CoreAudio CoreMIDI DiscRecording
+  dependencies:     juce_audio_basics, juce_events
+  OSXFrameworks:    CoreAudio CoreMIDI AudioToolbox
   iOSFrameworks:    CoreAudio CoreMIDI AudioToolbox AVFoundation
-  linuxLibs:        asound
+  linuxPackages:    alsa
   mingwLibs:        winmm
 
  END_JUCE_MODULE_DECLARATION
@@ -55,7 +55,6 @@
 
 #include <juce_events/juce_events.h>
 #include <juce_audio_basics/juce_audio_basics.h>
-#include <juce_audio_formats/juce_audio_formats.h>
 
 //==============================================================================
 /** Config: JUCE_ASIO
@@ -118,21 +117,6 @@
  #endif
 #endif
 
-//==============================================================================
-/** Config: JUCE_USE_CDREADER
-    Enables the AudioCDReader class (on supported platforms).
-*/
-#ifndef JUCE_USE_CDREADER
- #define JUCE_USE_CDREADER 0
-#endif
-
-/** Config: JUCE_USE_CDBURNER
-    Enables the AudioCDBurner class (on supported platforms).
-*/
-#ifndef JUCE_USE_CDBURNER
- #define JUCE_USE_CDBURNER 0
-#endif
-
 //==============================================================================
 namespace juce
 {
@@ -145,8 +129,6 @@ namespace juce
 #include "midi_io/juce_MidiOutput.h"
 #include "sources/juce_AudioSourcePlayer.h"
 #include "sources/juce_AudioTransportSource.h"
-#include "audio_cd/juce_AudioCDBurner.h"
-#include "audio_cd/juce_AudioCDReader.h"
 #include "audio_io/juce_AudioDeviceManager.h"
 
 }
