@@ -434,7 +434,9 @@ var::var() noexcept : type (&VariantType_Void::instance) {}
 var::var (const VariantType& t) noexcept  : type (&t) {}
 var::~var() noexcept  { type->cleanUp (value); }
 
+#if JUCE_ALLOW_STATIC_NULL_VARIABLES
 const var var::null;
+#endif
 
 //==============================================================================
 var::var (const var& valueToCopy)  : type (valueToCopy.type)
@@ -582,7 +584,7 @@ const var& var::operator[] (const Identifier& propertyName) const
     if (DynamicObject* const o = getDynamicObject())
         return o->getProperty (propertyName);
 
-    return var::null;
+    return getNullVarRef();
 }
 
 const var& var::operator[] (const char* const propertyName) const
