@@ -68,10 +68,49 @@ File& File::operator= (File&& other) noexcept
 }
 #endif
 
+#if JUCE_ALLOW_STATIC_NULL_VARIABLES
 const File File::nonexistent;
-
+#endif
 
 //==============================================================================
+static String removeEllipsis (const String& path)
+{
+    // This will quickly find both /../ and /./ at the expense of a minor
+    // false-positive performance hit when path elements end in a dot.
+   #if JUCE_WINDOWS
+    if (path.contains (".\\"))
+   #else
+    if (path.contains ("./"))
+   #endif
+    {
+        StringArray toks;
+        toks.addTokens (path, File::separatorString, StringRef());
+        bool anythingChanged = false;
+
+        for (int i = 1; i < toks.size(); ++i)
+        {
+            const String& t = toks[i];
+
+            if (t == ".." && toks[i - 1] != "..")
+            {
+                anythingChanged = true;
+                toks.removeRange (i - 1, 2);
+                i = jmax (0, i - 2);
+            }
+            else if (t == ".")
+            {
+                anythingChanged = true;
+                toks.remove (i--);
+            }
+        }
+
+        if (anythingChanged)
+            return toks.joinIntoString (File::separatorString);
+    }
+
+    return path;
+}
+
 String File::parseAbsolutePath (const String& p)
 {
     if (p.isEmpty())
@@ -79,7 +118,7 @@ String File::parseAbsolutePath (const String& p)
 
 #if JUCE_WINDOWS
     // Windows..
-    String path (p.replaceCharacter ('/', '\\'));
+    String path (removeEllipsis (p.replaceCharacter ('/', '\\')));
 
     if (path.startsWithChar (separator))
     {
@@ -118,7 +157,7 @@ String File::parseAbsolutePath (const String& p)
     // If that's why you've ended up here, use File::getChildFile() to build your paths instead.
     jassert ((! p.containsChar ('\\')) || (p.indexOfChar ('/') >= 0 && p.indexOfChar ('/') < p.indexOfChar ('\\')));
 
-    String path (p);
+    String path (removeEllipsis (p));
 
     if (path.startsWithChar ('~'))
     {
@@ -263,6 +302,21 @@ bool File::copyFileTo (const File& newFile) const
             || (exists() && newFile.deleteFile() && copyInternal (newFile));
 }
 
+bool File::replaceFileIn (const File& newFile) const
+{
+    if (newFile.fullPath == fullPath)
+        return true;
+
+    if (! newFile.exists())
+        return moveFileTo (newFile);
+
+    if (! replaceInternal (newFile))
+        return false;
+
+    deleteFile();
+    return true;
+}
+
 bool File::copyDirectoryTo (const File& newDirectory) const
 {
     if (isDirectory() && newDirectory.createDirectory())
@@ -347,62 +401,69 @@ int64 File::hashCode64() const  { return fullPath.hashCode64(); }
 //==============================================================================
 bool File::isAbsolutePath (StringRef path)
 {
-    return path.text[0] == separator
+    const juce_wchar firstChar = *(path.text);
+
+    return firstChar == separator
            #if JUCE_WINDOWS
-            || (path.isNotEmpty() && path.text[1] == ':');
+            || (firstChar != 0 && path.text[1] == ':');
            #else
-            || path.text[0] == '~';
+            || firstChar == '~';
            #endif
 }
 
 File File::getChildFile (StringRef relativePath) const
 {
-    if (isAbsolutePath (relativePath))
-        return File (String (relativePath.text));
+    String::CharPointerType r = relativePath.text;
 
-    if (relativePath[0] != '.')
-        return File (addTrailingSeparator (fullPath) + relativePath);
+    if (isAbsolutePath (r))
+        return File (String (r));
+
+   #if JUCE_WINDOWS
+    if (r.indexOf ((juce_wchar) '/') >= 0)
+        return getChildFile (String (r).replaceCharacter ('/', '\\'));
+   #endif
 
     String path (fullPath);
 
-    // It's relative, so remove any ../ or ./ bits at the start..
-   #if JUCE_WINDOWS
-    if (relativePath.text.indexOf ((juce_wchar) '/') >= 0)
-        return getChildFile (String (relativePath.text).replaceCharacter ('/', '\\'));
-   #endif
-
-    while (relativePath[0] == '.')
+    while (*r == '.')
     {
-        const juce_wchar secondChar = relativePath[1];
+        String::CharPointerType lastPos = r;
+        const juce_wchar secondChar = *++r;
 
-        if (secondChar == '.')
+        if (secondChar == '.') // remove "../"
         {
-            const juce_wchar thirdChar = relativePath[2];
+            const juce_wchar thirdChar = *++r;
 
-            if (thirdChar == 0 || thirdChar == separator)
+            if (thirdChar == separator || thirdChar == 0)
             {
                 const int lastSlash = path.lastIndexOfChar (separator);
                 if (lastSlash >= 0)
                     path = path.substring (0, lastSlash);
 
-                relativePath = relativePath.text + (thirdChar == 0 ? 2 : 3);
+                while (*r == separator) // ignore duplicate slashes
+                    ++r;
             }
             else
             {
+                r = lastPos;
                 break;
             }
         }
-        else if (secondChar == separator)
+        else if (secondChar == separator || secondChar == 0)  // remove "./"
         {
-            relativePath = relativePath.text + 2;
+            while (*r == separator) // ignore duplicate slashes
+                ++r;
         }
         else
         {
+            r = lastPos;
             break;
         }
     }
 
-    return File (addTrailingSeparator (path) + relativePath);
+    path = addTrailingSeparator (path);
+    path.appendCharPointer (r);
+    return File (path);
 }
 
 File File::getSiblingFile (StringRef fileName) const
@@ -922,16 +983,16 @@ bool File::createSymbolicLink (const File& linkFileToCreate, bool overwriteExist
 }
 
 //==============================================================================
-MemoryMappedFile::MemoryMappedFile (const File& file, MemoryMappedFile::AccessMode mode)
+MemoryMappedFile::MemoryMappedFile (const File& file, MemoryMappedFile::AccessMode mode, bool exclusive)
     : address (nullptr), range (0, file.getSize()), fileHandle (0)
 {
-    openInternal (file, mode);
+    openInternal (file, mode, exclusive);
 }
 
-MemoryMappedFile::MemoryMappedFile (const File& file, const Range<int64>& fileRange, AccessMode mode)
+MemoryMappedFile::MemoryMappedFile (const File& file, const Range<int64>& fileRange, AccessMode mode, bool exclusive)
     : address (nullptr), range (fileRange.getIntersectionWith (Range<int64> (0, file.getSize()))), fileHandle (0)
 {
-    openInternal (file, mode);
+    openInternal (file, mode, exclusive);
 }
 
 
@@ -950,9 +1011,9 @@ public:
         const File home (File::getSpecialLocation (File::userHomeDirectory));
         const File temp (File::getSpecialLocation (File::tempDirectory));
 
-        expect (! File::nonexistent.exists());
-        expect (! File::nonexistent.existsAsFile());
-        expect (! File::nonexistent.isDirectory());
+        expect (! File().exists());
+        expect (! File().existsAsFile());
+        expect (! File().isDirectory());
        #if ! JUCE_WINDOWS
         expect (File("/").isDirectory());
        #endif
@@ -1020,6 +1081,19 @@ public:
         expect (tempFile.getSiblingFile ("foo").isAChildOf (temp));
         expect (tempFile.hasWriteAccess());
 
+        expect (home.getChildFile (".") == home);
+        expect (home.getChildFile ("..") == home.getParentDirectory());
+        expect (home.getChildFile (".xyz").getFileName() == ".xyz");
+        expect (home.getChildFile ("..xyz").getFileName() == "..xyz");
+        expect (home.getChildFile ("...xyz").getFileName() == "...xyz");
+        expect (home.getChildFile ("./xyz") == home.getChildFile ("xyz"));
+        expect (home.getChildFile ("././xyz") == home.getChildFile ("xyz"));
+        expect (home.getChildFile ("../xyz") == home.getParentDirectory().getChildFile ("xyz"));
+        expect (home.getChildFile (".././xyz") == home.getParentDirectory().getChildFile ("xyz"));
+        expect (home.getChildFile (".././xyz/./abc") == home.getParentDirectory().getChildFile ("xyz/abc"));
+        expect (home.getChildFile ("./../xyz") == home.getParentDirectory().getChildFile ("xyz"));
+        expect (home.getChildFile ("a1/a2/a3/./../../a4") == home.getChildFile ("a1/a4"));
+
         {
             FileOutputStream fo (tempFile);
             fo.write ("0123456789", 10);
