@@ -28,76 +28,110 @@
 
 namespace
 {
-    inline size_t bitToIndex (const int bit) noexcept   { return (size_t) (bit >> 5); }
-    inline uint32 bitToMask  (const int bit) noexcept   { return (uint32) 1 << (bit & 31); }
+    inline uint32 bitToMask  (const int bit) noexcept           { return (uint32) 1 << (bit & 31); }
+    inline size_t bitToIndex (const int bit) noexcept           { return (size_t) (bit >> 5); }
+    inline size_t sizeNeededToHold (int highestBit) noexcept    { return (size_t) (highestBit >> 5) + 1; }
+
+    inline int highestBitInInt (uint32 n) noexcept
+    {
+        jassert (n != 0); // (the built-in functions may not work for n = 0)
+
+      #if JUCE_GCC || JUCE_CLANG
+        return 31 - __builtin_clz (n);
+      #elif JUCE_MSVC
+        unsigned long highest;
+        _BitScanReverse (&highest, n);
+        return (int) highest;
+      #else
+        n |= (n >> 1);
+        n |= (n >> 2);
+        n |= (n >> 4);
+        n |= (n >> 8);
+        n |= (n >> 16);
+        return countBitsInInt32 (n >> 1);
+      #endif
+    }
 }
 
 //==============================================================================
 BigInteger::BigInteger()
-    : numValues (4),
+    : allocatedSize (numPreallocatedInts),
       highestBit (-1),
       negative (false)
 {
-    values.calloc (numValues + 1);
+    for (int i = 0; i < numPreallocatedInts; ++i)
+        preallocated[i] = 0;
 }
 
 BigInteger::BigInteger (const int32 value)
-    : numValues (4),
+    : allocatedSize (numPreallocatedInts),
       highestBit (31),
       negative (value < 0)
 {
-    values.calloc (numValues + 1);
-    values[0] = (uint32) abs (value);
+    preallocated[0] = (uint32) std::abs (value);
+
+    for (int i = 1; i < numPreallocatedInts; ++i)
+        preallocated[i] = 0;
+
     highestBit = getHighestBit();
 }
 
 BigInteger::BigInteger (const uint32 value)
-    : numValues (4),
+    : allocatedSize (numPreallocatedInts),
       highestBit (31),
       negative (false)
 {
-    values.calloc (numValues + 1);
-    values[0] = value;
+    preallocated[0] = value;
+
+    for (int i = 1; i < numPreallocatedInts; ++i)
+        preallocated[i] = 0;
+
     highestBit = getHighestBit();
 }
 
 BigInteger::BigInteger (int64 value)
-    : numValues (4),
+    : allocatedSize (numPreallocatedInts),
       highestBit (63),
       negative (value < 0)
 {
-    values.calloc (numValues + 1);
-
     if (value < 0)
         value = -value;
 
-    values[0] = (uint32) value;
-    values[1] = (uint32) (value >> 32);
+    preallocated[0] = (uint32) value;
+    preallocated[1] = (uint32) (value >> 32);
+
+    for (int i = 2; i < numPreallocatedInts; ++i)
+        preallocated[i] = 0;
+
     highestBit = getHighestBit();
 }
 
 BigInteger::BigInteger (const BigInteger& other)
-    : numValues ((size_t) jmax ((size_t) 4, bitToIndex (other.highestBit) + 1)),
+    : allocatedSize (other.allocatedSize),
       highestBit (other.getHighestBit()),
       negative (other.negative)
 {
-    values.malloc (numValues + 1);
-    memcpy (values, other.values, sizeof (uint32) * (numValues + 1));
+    if (allocatedSize > numPreallocatedInts)
+        heapAllocation.malloc (allocatedSize);
+
+    memcpy (getValues(), other.getValues(), sizeof (uint32) * allocatedSize);
 }
 
 #if JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
 BigInteger::BigInteger (BigInteger&& other) noexcept
-    : values (static_cast<HeapBlock<uint32>&&> (other.values)),
-      numValues (other.numValues),
+    : heapAllocation (static_cast<HeapBlock<uint32>&&> (other.heapAllocation)),
+      allocatedSize (other.allocatedSize),
       highestBit (other.highestBit),
       negative (other.negative)
 {
+    memcpy (preallocated, other.preallocated, sizeof (preallocated));
 }
 
 BigInteger& BigInteger::operator= (BigInteger&& other) noexcept
 {
-    values = static_cast<HeapBlock<uint32>&&> (other.values);
-    numValues = other.numValues;
+    heapAllocation = static_cast<HeapBlock<uint32>&&> (other.heapAllocation);
+    memcpy (preallocated, other.preallocated, sizeof (preallocated));
+    allocatedSize = other.allocatedSize;
     highestBit = other.highestBit;
     negative = other.negative;
     return *this;
@@ -110,8 +144,11 @@ BigInteger::~BigInteger()
 
 void BigInteger::swapWith (BigInteger& other) noexcept
 {
-    values.swapWith (other.values);
-    std::swap (numValues, other.numValues);
+    for (int i = 0; i < numPreallocatedInts; ++i)
+        std::swap (preallocated[i], other.preallocated[i]);
+
+    heapAllocation.swapWith (other.heapAllocation);
+    std::swap (allocatedSize, other.allocatedSize);
     std::swap (highestBit, other.highestBit);
     std::swap (negative, other.negative);
 }
@@ -121,44 +158,70 @@ BigInteger& BigInteger::operator= (const BigInteger& other)
     if (this != &other)
     {
         highestBit = other.getHighestBit();
-        jassert (other.numValues >= 4);
-        numValues = (size_t) jmax ((size_t) 4, bitToIndex (highestBit) + 1);
+        const size_t newAllocatedSize = (size_t) jmax ((size_t) numPreallocatedInts, sizeNeededToHold (highestBit));
+
+        if (newAllocatedSize <= numPreallocatedInts)
+            heapAllocation.free();
+        else if (newAllocatedSize != allocatedSize)
+            heapAllocation.malloc (newAllocatedSize);
+
+        allocatedSize = newAllocatedSize;
+
+        memcpy (getValues(), other.getValues(), sizeof (uint32) * allocatedSize);
         negative = other.negative;
-        values.malloc (numValues + 1);
-        memcpy (values, other.values, sizeof (uint32) * (numValues + 1));
     }
 
     return *this;
 }
 
-void BigInteger::ensureSize (const size_t numVals)
+uint32* BigInteger::getValues() const noexcept
 {
-    if (numVals + 2 >= numValues)
-    {
-        size_t oldSize = numValues;
-        numValues = ((numVals + 2) * 3) / 2;
-        values.realloc (numValues + 1);
+    jassert (heapAllocation != nullptr || allocatedSize <= numPreallocatedInts);
 
-        while (oldSize < numValues)
-            values [oldSize++] = 0;
+    return heapAllocation != nullptr ? heapAllocation
+                                     : (uint32*) preallocated;
+}
+
+uint32* BigInteger::ensureSize (const size_t numVals)
+{
+    if (numVals > allocatedSize)
+    {
+        size_t oldSize = allocatedSize;
+        allocatedSize = ((numVals + 2) * 3) / 2;
+
+        if (heapAllocation == nullptr)
+        {
+            heapAllocation.calloc (allocatedSize);
+            memcpy (heapAllocation, preallocated, sizeof (uint32) * numPreallocatedInts);
+        }
+        else
+        {
+            heapAllocation.realloc (allocatedSize);
+
+            for (uint32* values = getValues(); oldSize < allocatedSize; ++oldSize)
+                values[oldSize] = 0;
+        }
     }
+
+    return getValues();
 }
 
 //==============================================================================
 bool BigInteger::operator[] (const int bit) const noexcept
 {
     return bit <= highestBit && bit >= 0
-             && ((values [bitToIndex (bit)] & bitToMask (bit)) != 0);
+             && ((getValues() [bitToIndex (bit)] & bitToMask (bit)) != 0);
 }
 
 int BigInteger::toInteger() const noexcept
 {
-    const int n = (int) (values[0] & 0x7fffffff);
+    const int n = (int) (getValues()[0] & 0x7fffffff);
     return negative ? -n : n;
 }
 
 int64 BigInteger::toInt64() const noexcept
 {
+    const uint32* values = getValues();
     const int64 n = (((int64) (values[1] & 0x7fffffff)) << 32) | values[0];
     return negative ? -n : n;
 }
@@ -167,13 +230,12 @@ BigInteger BigInteger::getBitRange (int startBit, int numBits) const
 {
     BigInteger r;
     numBits = jmin (numBits, getHighestBit() + 1 - startBit);
-    r.ensureSize ((size_t) bitToIndex (numBits));
+    uint32* const destValues = r.ensureSize (sizeNeededToHold (numBits));
     r.highestBit = numBits;
 
-    int i = 0;
-    while (numBits > 0)
+    for (int i = 0; numBits > 0;)
     {
-        r.values[i++] = getBitRangeAsInt (startBit, (int) jmin (32, numBits));
+        destValues[i++] = getBitRangeAsInt (startBit, (int) jmin (32, numBits));
         numBits -= 32;
         startBit += 32;
     }
@@ -198,6 +260,7 @@ uint32 BigInteger::getBitRangeAsInt (const int startBit, int numBits) const noex
     const size_t pos = bitToIndex (startBit);
     const int offset = startBit & 31;
     const int endSpace = 32 - numBits;
+    const uint32* values = getValues();
 
     uint32 n = ((uint32) values [pos]) >> offset;
 
@@ -223,20 +286,15 @@ void BigInteger::setBitRangeAsInt (const int startBit, int numBits, uint32 value
 }
 
 //==============================================================================
-void BigInteger::clear()
+void BigInteger::clear() noexcept
 {
-    if (numValues > 16)
-    {
-        numValues = 4;
-        values.calloc (numValues + 1);
-    }
-    else
-    {
-        values.clear (numValues + 1);
-    }
-
+    heapAllocation.free();
+    allocatedSize = numPreallocatedInts;
     highestBit = -1;
     negative = false;
+
+    for (int i = 0; i < numPreallocatedInts; ++i)
+        preallocated[i] = 0;
 }
 
 void BigInteger::setBit (const int bit)
@@ -245,11 +303,11 @@ void BigInteger::setBit (const int bit)
     {
         if (bit > highestBit)
         {
-            ensureSize (bitToIndex (bit));
+            ensureSize (sizeNeededToHold (bit));
             highestBit = bit;
         }
 
-        values [bitToIndex (bit)] |= bitToMask (bit);
+        getValues() [bitToIndex (bit)] |= bitToMask (bit);
     }
 }
 
@@ -264,7 +322,12 @@ void BigInteger::setBit (const int bit, const bool shouldBeSet)
 void BigInteger::clearBit (const int bit) noexcept
 {
     if (bit >= 0 && bit <= highestBit)
-        values [bitToIndex (bit)] &= ~bitToMask (bit);
+    {
+        getValues() [bitToIndex (bit)] &= ~bitToMask (bit);
+
+        if (bit == highestBit)
+            highestBit = getHighestBit();
+    }
 }
 
 void BigInteger::setRange (int startBit, int numBits, const bool shouldBeSet)
@@ -307,35 +370,16 @@ void BigInteger::negate() noexcept
     negative = (! negative) && ! isZero();
 }
 
-#if JUCE_USE_MSVC_INTRINSICS && ! defined (__INTEL_COMPILER)
+#if JUCE_MSVC && ! defined (__INTEL_COMPILER)
  #pragma intrinsic (_BitScanReverse)
 #endif
 
-inline static int highestBitInInt (uint32 n) noexcept
-{
-    jassert (n != 0); // (the built-in functions may not work for n = 0)
-
-  #if JUCE_GCC
-    return 31 - __builtin_clz (n);
-  #elif JUCE_USE_MSVC_INTRINSICS
-    unsigned long highest;
-    _BitScanReverse (&highest, n);
-    return (int) highest;
-  #else
-    n |= (n >> 1);
-    n |= (n >> 2);
-    n |= (n >> 4);
-    n |= (n >> 8);
-    n |= (n >> 16);
-    return countBitsInInt32 (n >> 1);
-  #endif
-}
-
 int BigInteger::countNumberOfSetBits() const noexcept
 {
     int total = 0;
+    const uint32* values = getValues();
 
-    for (int i = (int) bitToIndex (highestBit) + 1; --i >= 0;)
+    for (int i = (int) sizeNeededToHold (highestBit); --i >= 0;)
         total += countNumberOfBits (values[i]);
 
     return total;
@@ -343,19 +387,19 @@ int BigInteger::countNumberOfSetBits() const noexcept
 
 int BigInteger::getHighestBit() const noexcept
 {
-    for (int i = (int) bitToIndex (highestBit + 1); i >= 0; --i)
-    {
-        const uint32 n = values[i];
+    const uint32* values = getValues();
 
-        if (n != 0)
+    for (int i = (int) bitToIndex (highestBit); i >= 0; --i)
+        if (uint32 n = values[i])
             return highestBitInInt (n) + (i << 5);
-    }
 
     return -1;
 }
 
 int BigInteger::findNextSetBit (int i) const noexcept
 {
+    const uint32* values = getValues();
+
     for (; i <= highestBit; ++i)
         if ((values [bitToIndex (i)] & bitToMask (i)) != 0)
             return i;
@@ -365,6 +409,8 @@ int BigInteger::findNextSetBit (int i) const noexcept
 
 int BigInteger::findNextClearBit (int i) const noexcept
 {
+    const uint32* values = getValues();
+
     for (; i <= highestBit; ++i)
         if ((values [bitToIndex (i)] & bitToMask (i)) == 0)
             break;
@@ -375,6 +421,9 @@ int BigInteger::findNextClearBit (int i) const noexcept
 //==============================================================================
 BigInteger& BigInteger::operator+= (const BigInteger& other)
 {
+    if (this == &other)
+        return operator+= (BigInteger (other));
+
     if (other.isNegative())
         return operator-= (-other);
 
@@ -385,34 +434,30 @@ BigInteger& BigInteger::operator+= (const BigInteger& other)
             BigInteger temp (*this);
             temp.negate();
             *this = other;
-            operator-= (temp);
+            *this -= temp;
         }
         else
         {
             negate();
-            operator-= (other);
+            *this -= other;
             negate();
         }
     }
     else
     {
-        if (other.highestBit > highestBit)
-            highestBit = other.highestBit;
-
-        ++highestBit;
-
-        const size_t numInts = bitToIndex (highestBit) + 1;
-        ensureSize (numInts);
+        highestBit = jmax (highestBit, other.highestBit) + 1;
 
+        const size_t numInts = sizeNeededToHold (highestBit);
+        uint32* const values = ensureSize (numInts);
+        const uint32* const otherValues = other.getValues();
         int64 remainder = 0;
 
-        for (size_t i = 0; i <= numInts; ++i)
+        for (size_t i = 0; i < numInts; ++i)
         {
-            if (i < numValues)
-                remainder += values[i];
+            remainder += values[i];
 
-            if (i < other.numValues)
-                remainder += other.values[i];
+            if (i < other.allocatedSize)
+                remainder += otherValues[i];
 
             values[i] = (uint32) remainder;
             remainder >>= 32;
@@ -427,36 +472,43 @@ BigInteger& BigInteger::operator+= (const BigInteger& other)
 
 BigInteger& BigInteger::operator-= (const BigInteger& other)
 {
+    if (this == &other)
+    {
+        clear();
+        return *this;
+    }
+
     if (other.isNegative())
         return operator+= (-other);
 
-    if (! isNegative())
-    {
-        if (compareAbsolute (other) < 0)
-        {
-            BigInteger temp (other);
-            swapWith (temp);
-            operator-= (temp);
-            negate();
-            return *this;
-        }
-    }
-    else
+    if (isNegative())
     {
         negate();
-        operator+= (other);
+        *this += other;
         negate();
         return *this;
     }
 
-    const size_t numInts = bitToIndex (highestBit) + 1;
-    const size_t maxOtherInts = bitToIndex (other.highestBit) + 1;
+    if (compareAbsolute (other) < 0)
+    {
+        BigInteger temp (other);
+        swapWith (temp);
+        *this -= temp;
+        negate();
+        return *this;
+    }
+
+    const size_t numInts = sizeNeededToHold (getHighestBit());
+    const size_t maxOtherInts = sizeNeededToHold (other.getHighestBit());
+    jassert (numInts >= maxOtherInts);
+    uint32* const values = getValues();
+    const uint32* const otherValues = other.getValues();
     int64 amountToSubtract = 0;
 
-    for (size_t i = 0; i <= numInts; ++i)
+    for (size_t i = 0; i < numInts; ++i)
     {
-        if (i <= maxOtherInts)
-            amountToSubtract += (int64) other.values[i];
+        if (i < maxOtherInts)
+            amountToSubtract += (int64) otherValues[i];
 
         if (values[i] >= amountToSubtract)
         {
@@ -471,34 +523,60 @@ BigInteger& BigInteger::operator-= (const BigInteger& other)
         }
     }
 
+    highestBit = getHighestBit();
     return *this;
 }
 
 BigInteger& BigInteger::operator*= (const BigInteger& other)
 {
-    BigInteger total;
-    highestBit = getHighestBit();
+    if (this == &other)
+        return operator*= (BigInteger (other));
+
+    int n = getHighestBit();
+    int t = other.getHighestBit();
+
     const bool wasNegative = isNegative();
     setNegative (false);
 
-    for (int i = 0; i <= highestBit; ++i)
+    BigInteger total;
+    total.highestBit = n + t + 1;
+    uint32* const totalValues = total.ensureSize (sizeNeededToHold (total.highestBit) + 1);
+
+    n >>= 5;
+    t >>= 5;
+
+    BigInteger m (other);
+    m.setNegative (false);
+
+    const uint32* const mValues = m.getValues();
+    const uint32* const values = getValues();
+
+    for (int i = 0; i <= t; ++i)
     {
-        if (operator[](i))
+        uint32 c = 0;
+
+        for (int j = 0; j <= n; ++j)
         {
-            BigInteger n (other);
-            n.setNegative (false);
-            n <<= i;
-            total += n;
+            uint64 uv = (uint64) totalValues[i + j] + (uint64) values[j] * (uint64) mValues[i] + (uint64) c;
+            totalValues[i + j] = (uint32) uv;
+            c = uv >> 32;
         }
+
+        totalValues[i + n + 1] = c;
     }
 
+    total.highestBit = total.getHighestBit();
     total.setNegative (wasNegative ^ other.isNegative());
     swapWith (total);
+
     return *this;
 }
 
 void BigInteger::divideBy (const BigInteger& divisor, BigInteger& remainder)
 {
+    if (this == &divisor)
+        return divideBy (BigInteger (divisor), remainder);
+
     jassert (this != &remainder); // (can't handle passing itself in to get the remainder)
 
     const int divHB = divisor.getHighestBit();
@@ -550,17 +628,21 @@ BigInteger& BigInteger::operator/= (const BigInteger& other)
 
 BigInteger& BigInteger::operator|= (const BigInteger& other)
 {
+    if (this == &other)
+        return *this;
+
     // this operation doesn't take into account negative values..
     jassert (isNegative() == other.isNegative());
 
     if (other.highestBit >= 0)
     {
-        ensureSize (bitToIndex (other.highestBit));
+        uint32* const values = ensureSize (sizeNeededToHold (other.highestBit));
+        const uint32* const otherValues = other.getValues();
 
         int n = (int) bitToIndex (other.highestBit) + 1;
 
         while (--n >= 0)
-            values[n] |= other.values[n];
+            values[n] |= otherValues[n];
 
         if (other.highestBit > highestBit)
             highestBit = other.highestBit;
@@ -573,16 +655,22 @@ BigInteger& BigInteger::operator|= (const BigInteger& other)
 
 BigInteger& BigInteger::operator&= (const BigInteger& other)
 {
+    if (this == &other)
+        return *this;
+
     // this operation doesn't take into account negative values..
     jassert (isNegative() == other.isNegative());
 
-    int n = (int) numValues;
+    uint32* const values = getValues();
+    const uint32* const otherValues = other.getValues();
 
-    while (n > (int) other.numValues)
+    int n = (int) allocatedSize;
+
+    while (n > (int) other.allocatedSize)
         values[--n] = 0;
 
     while (--n >= 0)
-        values[n] &= other.values[n];
+        values[n] &= otherValues[n];
 
     if (other.highestBit < highestBit)
         highestBit = other.highestBit;
@@ -593,17 +681,24 @@ BigInteger& BigInteger::operator&= (const BigInteger& other)
 
 BigInteger& BigInteger::operator^= (const BigInteger& other)
 {
+    if (this == &other)
+    {
+        clear();
+        return *this;
+    }
+
     // this operation will only work with the absolute values
     jassert (isNegative() == other.isNegative());
 
     if (other.highestBit >= 0)
     {
-        ensureSize (bitToIndex (other.highestBit));
+        uint32* const values = ensureSize (sizeNeededToHold (other.highestBit));
+        const uint32* const otherValues = other.getValues();
 
         int n = (int) bitToIndex (other.highestBit) + 1;
 
         while (--n >= 0)
-            values[n] ^= other.values[n];
+            values[n] ^= otherValues[n];
 
         if (other.highestBit > highestBit)
             highestBit = other.highestBit;
@@ -644,15 +739,15 @@ BigInteger& BigInteger::operator>>= (const int numBits)              { shiftBits
 //==============================================================================
 int BigInteger::compare (const BigInteger& other) const noexcept
 {
-    if (isNegative() == other.isNegative())
+    const bool isNeg = isNegative();
+
+    if (isNeg == other.isNegative())
     {
         const int absComp = compareAbsolute (other);
-        return isNegative() ? -absComp : absComp;
-    }
-    else
-    {
-        return isNegative() ? -1 : 1;
+        return isNeg ? -absComp : absComp;
     }
+
+    return isNeg ? -1 : 1;
 }
 
 int BigInteger::compareAbsolute (const BigInteger& other) const noexcept
@@ -660,23 +755,24 @@ int BigInteger::compareAbsolute (const BigInteger& other) const noexcept
     const int h1 = getHighestBit();
     const int h2 = other.getHighestBit();
 
-    if (h1 > h2)
-        return 1;
-    else if (h1 < h2)
-        return -1;
+    if (h1 > h2) return 1;
+    if (h1 < h2) return -1;
 
-    for (int i = (int) bitToIndex (h1) + 1; --i >= 0;)
-        if (values[i] != other.values[i])
-            return (values[i] > other.values[i]) ? 1 : -1;
+    const uint32* const values = getValues();
+    const uint32* const otherValues = other.getValues();
+
+    for (int i = (int) bitToIndex (h1); i >= 0; --i)
+        if (values[i] != otherValues[i])
+            return values[i] > otherValues[i] ? 1 : -1;
 
     return 0;
 }
 
 bool BigInteger::operator== (const BigInteger& other) const noexcept    { return compare (other) == 0; }
 bool BigInteger::operator!= (const BigInteger& other) const noexcept    { return compare (other) != 0; }
-bool BigInteger::operator<  (const BigInteger& other) const noexcept    { return compare (other) < 0; }
+bool BigInteger::operator<  (const BigInteger& other) const noexcept    { return compare (other) <  0; }
 bool BigInteger::operator<= (const BigInteger& other) const noexcept    { return compare (other) <= 0; }
-bool BigInteger::operator>  (const BigInteger& other) const noexcept    { return compare (other) > 0; }
+bool BigInteger::operator>  (const BigInteger& other) const noexcept    { return compare (other) >  0; }
 bool BigInteger::operator>= (const BigInteger& other) const noexcept    { return compare (other) >= 0; }
 
 //==============================================================================
@@ -684,27 +780,27 @@ void BigInteger::shiftLeft (int bits, const int startBit)
 {
     if (startBit > 0)
     {
-        for (int i = highestBit + 1; --i >= startBit;)
-            setBit (i + bits, operator[] (i));
+        for (int i = highestBit; i >= startBit; --i)
+            setBit (i + bits, (*this) [i]);
 
         while (--bits >= 0)
             clearBit (bits + startBit);
     }
     else
     {
-        ensureSize (bitToIndex (highestBit + bits) + 1);
+        uint32* const values = ensureSize (sizeNeededToHold (highestBit + bits));
 
         const size_t wordsToMove = bitToIndex (bits);
-        size_t top = 1 + bitToIndex (highestBit);
+        size_t numOriginalInts = bitToIndex (highestBit);
         highestBit += bits;
 
         if (wordsToMove > 0)
         {
-            for (int i = (int) top; --i >= 0;)
-                values [(size_t) i + wordsToMove] = values [i];
+            for (int i = (int) numOriginalInts; i >= 0; --i)
+                values[(size_t) i + wordsToMove] = values[i];
 
             for (size_t j = 0; j < wordsToMove; ++j)
-                values [j] = 0;
+                values[j] = 0;
 
             bits &= 31;
         }
@@ -713,10 +809,10 @@ void BigInteger::shiftLeft (int bits, const int startBit)
         {
             const int invBits = 32 - bits;
 
-            for (size_t i = top + 1 + wordsToMove; --i > wordsToMove;)
-                values[i] = (values[i] << bits) | (values [i - 1] >> invBits);
+            for (size_t i = bitToIndex (highestBit); i > wordsToMove; --i)
+                values[i] = (values[i] << bits) | (values[i - 1] >> invBits);
 
-            values [wordsToMove] = values [wordsToMove] << bits;
+            values[wordsToMove] = values[wordsToMove] << bits;
         }
 
         highestBit = getHighestBit();
@@ -728,7 +824,7 @@ void BigInteger::shiftRight (int bits, const int startBit)
     if (startBit > 0)
     {
         for (int i = startBit; i <= highestBit; ++i)
-            setBit (i, operator[] (i + bits));
+            setBit (i, (*this) [i + bits]);
 
         highestBit = getHighestBit();
     }
@@ -743,15 +839,16 @@ void BigInteger::shiftRight (int bits, const int startBit)
             const size_t wordsToMove = bitToIndex (bits);
             size_t top = 1 + bitToIndex (highestBit) - wordsToMove;
             highestBit -= bits;
+            uint32* const values = getValues();
 
             if (wordsToMove > 0)
             {
                 size_t i;
                 for (i = 0; i < top; ++i)
-                    values [i] = values [i + wordsToMove];
+                    values[i] = values[i + wordsToMove];
 
                 for (i = 0; i < wordsToMove; ++i)
-                    values [top + i] = 0;
+                    values[top + i] = 0;
 
                 bits &= 31;
             }
@@ -759,10 +856,10 @@ void BigInteger::shiftRight (int bits, const int startBit)
             if (bits != 0)
             {
                 const int invBits = 32 - bits;
-
                 --top;
+
                 for (size_t i = 0; i < top; ++i)
-                    values[i] = (values[i] >> bits) | (values [i + 1] << invBits);
+                    values[i] = (values[i] >> bits) | (values[i + 1] << invBits);
 
                 values[top] = (values[top] >> bits);
             }
@@ -803,7 +900,7 @@ BigInteger BigInteger::findGreatestCommonDivisor (BigInteger n) const
 
     while (! n.isZero())
     {
-        if (abs (m.getHighestBit() - n.getHighestBit()) <= 16)
+        if (std::abs (m.getHighestBit() - n.getHighestBit()) <= 16)
             return simpleGCD (&m, &n);
 
         BigInteger temp2;
@@ -818,25 +915,128 @@ BigInteger BigInteger::findGreatestCommonDivisor (BigInteger n) const
 
 void BigInteger::exponentModulo (const BigInteger& exponent, const BigInteger& modulus)
 {
+    *this %= modulus;
     BigInteger exp (exponent);
     exp %= modulus;
 
-    BigInteger value (1);
-    swapWith (value);
-    value %= modulus;
-
-    while (! exp.isZero())
+    if (modulus.getHighestBit() <= 32 || modulus % 2 == 0)
     {
-        if (exp [0])
-        {
-            operator*= (value);
-            operator%= (modulus);
-        }
+        BigInteger a (*this);
 
-        value *= value;
-        value %= modulus;
-        exp >>= 1;
+        const int n = exp.getHighestBit();
+
+        for (int i = n; --i >= 0;)
+        {
+            *this *= *this;
+
+            if (exp[i])
+                *this *= a;
+
+            if (compareAbsolute (modulus) >= 0)
+                *this %= modulus;
+        }
     }
+    else
+    {
+        const int Rfactor = modulus.getHighestBit() + 1;
+        BigInteger R (1);
+        R.shiftLeft (Rfactor, 0);
+
+        BigInteger R1, m1, g;
+        g.extendedEuclidean (modulus, R, m1, R1);
+
+        if (! g.isOne())
+        {
+            BigInteger a (*this);
+
+            for (int i = exp.getHighestBit(); --i >= 0;)
+            {
+                *this *= *this;
+
+                if (exp[i])
+                    *this *= a;
+
+                if (compareAbsolute (modulus) >= 0)
+                    *this %= modulus;
+            }
+        }
+        else
+        {
+            BigInteger am (((*this) * R) % modulus);
+            BigInteger xm (am);
+            BigInteger um (R % modulus);
+
+            for (int i = exp.getHighestBit(); --i >= 0;)
+            {
+                xm.montgomeryMultiplication (xm, modulus, m1, Rfactor);
+
+                if (exp[i])
+                    xm.montgomeryMultiplication (am, modulus, m1, Rfactor);
+            }
+
+            xm.montgomeryMultiplication (1, modulus, m1, Rfactor);
+            swapWith (xm);
+        }
+    }
+}
+
+void BigInteger::montgomeryMultiplication (const BigInteger& other, const BigInteger& modulus,
+                                           const BigInteger& modulusp, const int k)
+{
+    *this *= other;
+
+    BigInteger t (*this);
+
+    setRange (k, highestBit - k + 1, false);
+    *this *= modulusp;
+
+    setRange (k, highestBit - k + 1, false);
+    *this *= modulus;
+    *this += t;
+    shiftRight (k, 0);
+
+    if (compare (modulus) >= 0)
+        *this -= modulus;
+    else if (isNegative())
+        *this += modulus;
+}
+
+void BigInteger::extendedEuclidean (const BigInteger& a, const BigInteger& b,
+                                    BigInteger& x, BigInteger& y)
+{
+    BigInteger p(a), q(b), gcd(1);
+
+    Array<BigInteger> tempValues;
+
+    while (! q.isZero())
+    {
+        tempValues.add (p / q);
+        gcd = q;
+        q = p % q;
+        p = gcd;
+    }
+
+    x.clear();
+    y = 1;
+
+    for (int i = 1; i < tempValues.size(); ++i)
+    {
+        const BigInteger& v = tempValues.getReference (tempValues.size() - i - 1);
+
+        if ((i & 1) != 0)
+            x += y * v;
+        else
+            y += x * v;
+    }
+
+    if (gcd.compareAbsolute (y * b - x * a) != 0)
+    {
+        x.negate();
+        x.swapWith (y);
+        x.negate();
+    }
+
+    swapWith (gcd);
 }
 
 void BigInteger::inverseModulo (const BigInteger& modulus)
@@ -848,22 +1048,19 @@ void BigInteger::inverseModulo (const BigInteger& modulus)
     }
 
     if (isNegative() || compareAbsolute (modulus) >= 0)
-        operator%= (modulus);
+        *this %= modulus;
 
     if (isOne())
         return;
 
-    if (! (*this)[0])
+    if (findGreatestCommonDivisor (modulus) != 1)
     {
-        // not invertible
-        clear();
+        clear();  // not invertible!
         return;
     }
 
-    BigInteger a1 (modulus);
-    BigInteger a2 (*this);
-    BigInteger b1 (modulus);
-    BigInteger b2 (1);
+    BigInteger a1 (modulus), a2 (*this);
+    BigInteger b1 (modulus), b2 (1);
 
     while (! a2.isOne())
     {
@@ -964,8 +1161,8 @@ void BigInteger::parseString (StringRef text, const int base)
 
             if (((uint32) digit) < (uint32) base)
             {
-                operator<<= (bits);
-                operator+= (digit);
+                *this <<= bits;
+                *this += digit;
             }
             else if (c == 0)
             {
@@ -983,8 +1180,8 @@ void BigInteger::parseString (StringRef text, const int base)
 
             if (c >= '0' && c <= '9')
             {
-                operator*= (ten);
-                operator+= ((int) (c - '0'));
+                *this *= ten;
+                *this += (int) (c - '0');
             }
             else if (c == 0)
             {
@@ -998,6 +1195,7 @@ MemoryBlock BigInteger::toMemoryBlock() const
 {
     const int numBytes = (getHighestBit() + 8) >> 3;
     MemoryBlock mb ((size_t) numBytes);
+    const uint32* const values = getValues();
 
     for (int i = 0; i < numBytes; ++i)
         mb[i] = (char) ((values[i / 4] >> ((i & 3) * 8)) & 0xff);
@@ -1008,14 +1206,13 @@ MemoryBlock BigInteger::toMemoryBlock() const
 void BigInteger::loadFromMemoryBlock (const MemoryBlock& data)
 {
     const size_t numBytes = data.getSize();
-    numValues = 1 + (numBytes / sizeof (uint32));
-    values.malloc (numValues + 1);
+    const size_t numInts = 1 + (numBytes / sizeof (uint32));
+    uint32* const values = ensureSize (numInts);
 
-    for (int i = 0; i < (int) numValues - 1; ++i)
+    for (int i = 0; i < (int) numInts - 1; ++i)
         values[i] = (uint32) ByteOrder::littleEndianInt (addBytesToPointer (data.getData(), sizeof (uint32) * (size_t) i));
 
-    values[numValues - 1] = 0;
-    values[numValues] = 0;
+    values[numInts - 1] = 0;
 
     for (int i = (int) (numBytes & ~3u); i < (int) numBytes; ++i)
         this->setBitRangeAsInt (i << 3, 8, (uint32) data [i]);
@@ -1024,6 +1221,76 @@ void BigInteger::loadFromMemoryBlock (const MemoryBlock& data)
     highestBit = getHighestBit();
 }
 
+//==============================================================================
+void writeLittleEndianBitsInBuffer (void* buffer, uint32 startBit, uint32 numBits, uint32 value) noexcept
+{
+    jassert (buffer != nullptr);
+    jassert (numBits > 0 && numBits <= 32);
+    jassert (numBits == 32 || (value >> numBits) == 0);
+
+    uint8* data = static_cast<uint8*> (buffer) + startBit / 8;
+
+    if (const uint32 offset = (startBit & 7))
+    {
+        const uint32 bitsInByte = 8 - offset;
+        const uint8 current = *data;
+
+        if (bitsInByte >= numBits)
+        {
+            *data = (uint8) ((current & ~(((1u << numBits) - 1u) << offset)) | (value << offset));
+            return;
+        }
+
+        *data++ = current ^ (uint8) (((value << offset) ^ current) & (((1u << bitsInByte) - 1u) << offset));
+        numBits -= bitsInByte;
+        value >>= bitsInByte;
+    }
+
+    while (numBits >= 8)
+    {
+        *data++ = (uint8) value;
+        value >>= 8;
+        numBits -= 8;
+    }
+
+    if (numBits > 0)
+        *data = (uint8) ((*data & (0xff << numBits)) | value);
+}
+
+uint32 readLittleEndianBitsInBuffer (const void* buffer, uint32 startBit, uint32 numBits) noexcept
+{
+    jassert (buffer != nullptr);
+    jassert (numBits > 0 && numBits <= 32);
+
+    uint32 result = 0;
+    uint32 bitsRead = 0;
+    const uint8* data = static_cast<const uint8*> (buffer) + startBit / 8;
+
+    if (const uint32 offset = (startBit & 7))
+    {
+        const uint32 bitsInByte = 8 - offset;
+        result = (*data >> offset);
+
+        if (bitsInByte >= numBits)
+            return result & ((1u << numBits) - 1u);
+
+        numBits -= bitsInByte;
+        bitsRead += bitsInByte;
+        ++data;
+    }
+
+    while (numBits >= 8)
+    {
+        result |= (((uint32) *data++) << bitsRead);
+        bitsRead += 8;
+        numBits -= 8;
+    }
+
+    if (numBits > 0)
+        result |= ((*data & ((1u << numBits) - 1u)) << bitsRead);
+
+    return result;
+}
 
 //==============================================================================
 //==============================================================================
@@ -1046,33 +1313,64 @@ public:
 
     void runTest() override
     {
-        beginTest ("BigInteger");
-
-        Random r = getRandom();
-
-        expect (BigInteger().isZero());
-        expect (BigInteger(1).isOne());
-
-        for (int j = 10000; --j >= 0;)
         {
-            BigInteger b1 (getBigRandom(r)),
-                       b2 (getBigRandom(r));
+            beginTest ("BigInteger");
 
-            BigInteger b3 = b1 + b2;
-            expect (b3 > b1 && b3 > b2);
-            expect (b3 - b1 == b2);
-            expect (b3 - b2 == b1);
+            Random r = getRandom();
 
-            BigInteger b4 = b1 * b2;
-            expect (b4 > b1 && b4 > b2);
-            expect (b4 / b1 == b2);
-            expect (b4 / b2 == b1);
+            expect (BigInteger().isZero());
+            expect (BigInteger(1).isOne());
 
-            // TODO: should add tests for other ops (although they also get pretty well tested in the RSA unit test)
+            for (int j = 10000; --j >= 0;)
+            {
+                BigInteger b1 (getBigRandom(r)),
+                           b2 (getBigRandom(r));
 
-            BigInteger b5;
-            b5.loadFromMemoryBlock (b3.toMemoryBlock());
-            expect (b3 == b5);
+                BigInteger b3 = b1 + b2;
+                expect (b3 > b1 && b3 > b2);
+                expect (b3 - b1 == b2);
+                expect (b3 - b2 == b1);
+
+                BigInteger b4 = b1 * b2;
+                expect (b4 > b1 && b4 > b2);
+                expect (b4 / b1 == b2);
+                expect (b4 / b2 == b1);
+                expect (((b4 << 1) >> 1) == b4);
+                expect (((b4 << 10) >> 10) == b4);
+                expect (((b4 << 100) >> 100) == b4);
+
+                // TODO: should add tests for other ops (although they also get pretty well tested in the RSA unit test)
+
+                BigInteger b5;
+                b5.loadFromMemoryBlock (b3.toMemoryBlock());
+                expect (b3 == b5);
+            }
+        }
+
+        {
+            beginTest ("Bit setting");
+
+            Random r = getRandom();
+            static uint8 test[2048];
+
+            for (int j = 100000; --j >= 0;)
+            {
+                uint32 offset = static_cast<uint32> (r.nextInt (200) + 10);
+                uint32 num = static_cast<uint32> (r.nextInt (32) + 1);
+                uint32 value = static_cast<uint32> (r.nextInt());
+
+                if (num < 32)
+                    value &= ((1u << num) - 1);
+
+                auto old1 = readLittleEndianBitsInBuffer (test, offset - 6, 6);
+                auto old2 = readLittleEndianBitsInBuffer (test, offset + num, 6);
+                writeLittleEndianBitsInBuffer (test, offset, num, value);
+                auto result = readLittleEndianBitsInBuffer (test, offset, num);
+
+                expect (result == value);
+                expect (old1 == readLittleEndianBitsInBuffer (test, offset - 6, 6));
+                expect (old2 == readLittleEndianBitsInBuffer (test, offset + num, 6));
+            }
         }
     }
 };
