@@ -147,7 +147,7 @@ public:
    #endif
 
     /** The raw value that this class operates on.
-        This is exposed publically in case you need to manipulate it directly
+        This is exposed publicly in case you need to manipulate it directly
         for performance reasons.
     */
     volatile Type value;
@@ -187,21 +187,11 @@ private:
 /*
     The following code is in the header so that the atomics can be inlined where possible...
 */
-#if JUCE_MAC && (JUCE_PPC || __GNUC__ < 4 || (__GNUC__ == 4 && __GNUC_MINOR__ < 2))
+#if JUCE_MAC && (__GNUC__ < 4 || (__GNUC__ == 4 && __GNUC_MINOR__ < 2))
   #define JUCE_ATOMICS_MAC_LEGACY 1      // Older OSX builds using gcc4.1 or earlier
 
-  #if JUCE_PPC
-    // None of these atomics are available for PPC or for iOS 3.1 or earlier!!
-    template <typename Type> static Type OSAtomicAdd64Barrier (Type b, volatile Type* a) noexcept  { jassertfalse; return *a += b; }
-    template <typename Type> static Type OSAtomicIncrement64Barrier (volatile Type* a) noexcept    { jassertfalse; return ++*a; }
-    template <typename Type> static Type OSAtomicDecrement64Barrier (volatile Type* a) noexcept    { jassertfalse; return --*a; }
-    template <typename Type> static bool OSAtomicCompareAndSwap64Barrier (Type old, Type newValue, volatile Type* value) noexcept
-        { jassertfalse; if (old == *value) { *value = newValue; return true; } return false; }
-    #define JUCE_64BIT_ATOMICS_UNAVAILABLE 1
-  #endif
-
 //==============================================================================
-#elif (JUCE_GCC || JUCE_CLANG) && ! JUCE_MSVC
+#elif JUCE_GCC || JUCE_CLANG
   #define JUCE_ATOMICS_GCC 1        // GCC with intrinsics
 
   #if JUCE_IOS || JUCE_ANDROID // (64-bit ops will compile but not link on these mobile OSes)
@@ -212,27 +202,17 @@ private:
 #else
   #define JUCE_ATOMICS_WINDOWS 1    // Windows with intrinsics
 
-  #if JUCE_USE_MSVC_INTRINSICS
-    #ifndef __INTEL_COMPILER
-     #pragma intrinsic (_InterlockedExchange, _InterlockedIncrement, _InterlockedDecrement, _InterlockedCompareExchange, \
-                        _InterlockedCompareExchange64, _InterlockedExchangeAdd, _ReadWriteBarrier)
-    #endif
-    #define juce_InterlockedExchange(a, b)              _InterlockedExchange(a, b)
-    #define juce_InterlockedIncrement(a)                _InterlockedIncrement(a)
-    #define juce_InterlockedDecrement(a)                _InterlockedDecrement(a)
-    #define juce_InterlockedExchangeAdd(a, b)           _InterlockedExchangeAdd(a, b)
-    #define juce_InterlockedCompareExchange(a, b, c)    _InterlockedCompareExchange(a, b, c)
-    #define juce_InterlockedCompareExchange64(a, b, c)  _InterlockedCompareExchange64(a, b, c)
-    #define juce_MemoryBarrier _ReadWriteBarrier
-  #else
-    long juce_InterlockedExchange (volatile long* a, long b) noexcept;
-    long juce_InterlockedIncrement (volatile long* a) noexcept;
-    long juce_InterlockedDecrement (volatile long* a) noexcept;
-    long juce_InterlockedExchangeAdd (volatile long* a, long b) noexcept;
-    long juce_InterlockedCompareExchange (volatile long* a, long b, long c) noexcept;
-    __int64 juce_InterlockedCompareExchange64 (volatile __int64* a, __int64 b, __int64 c) noexcept;
-    inline void juce_MemoryBarrier() noexcept  { long x = 0; juce_InterlockedIncrement (&x); }
+  #ifndef __INTEL_COMPILER
+   #pragma intrinsic (_InterlockedExchange, _InterlockedIncrement, _InterlockedDecrement, _InterlockedCompareExchange, \
+                      _InterlockedCompareExchange64, _InterlockedExchangeAdd, _ReadWriteBarrier)
   #endif
+  #define juce_InterlockedExchange(a, b)              _InterlockedExchange(a, b)
+  #define juce_InterlockedIncrement(a)                _InterlockedIncrement(a)
+  #define juce_InterlockedDecrement(a)                _InterlockedDecrement(a)
+  #define juce_InterlockedExchangeAdd(a, b)           _InterlockedExchangeAdd(a, b)
+  #define juce_InterlockedCompareExchange(a, b, c)    _InterlockedCompareExchange(a, b, c)
+  #define juce_InterlockedCompareExchange64(a, b, c)  _InterlockedCompareExchange64(a, b, c)
+  #define juce_MemoryBarrier _ReadWriteBarrier
 
   #if JUCE_64BIT
     #ifndef __INTEL_COMPILER
