@@ -62,6 +62,11 @@ namespace SocketHelpers
        #endif
     }
 
+    inline bool isValidPortNumber (int port) noexcept
+    {
+        return isPositiveAndBelow (port, 65536);
+    }
+
     template <typename Type>
     static bool setOption (const SocketHandle handle, int mode, int property, Type value) noexcept
     {
@@ -115,6 +120,7 @@ namespace SocketHelpers
         {
             // unblock any pending read requests
             ::shutdown (h, SHUT_RDWR);
+
             {
                 // see man-page of recv on linux about a race condition where the
                 // shutdown command is lost if the receiving thread does not have
@@ -133,37 +139,32 @@ namespace SocketHelpers
        #endif
     }
 
-    static bool bindSocket (const SocketHandle handle, const int port, const String& address) noexcept
+    static bool bindSocket (const SocketHandle handle, int port, const String& address) noexcept
     {
-        if (handle <= 0 || port < 0)
+        if (handle <= 0 || ! isValidPortNumber (port))
             return false;
 
-        struct sockaddr_in servTmpAddr;
-        zerostruct (servTmpAddr); // (can't use "= { 0 }" on this object because it's typedef'ed as a C struct)
-        servTmpAddr.sin_family = PF_INET;
-        servTmpAddr.sin_addr.s_addr = htonl (INADDR_ANY);
-        servTmpAddr.sin_port = htons ((uint16) port);
+        struct sockaddr_in addr;
+        zerostruct (addr); // (can't use "= { 0 }" on this object because it's typedef'ed as a C struct)
 
-       #if JUCE_WINDOWS
-        if (address.isNotEmpty())
-            servTmpAddr.sin_addr.s_addr = ::inet_addr (address.toUTF8());
-       #else
-        ignoreUnused (address);
-       #endif
+        addr.sin_family = PF_INET;
+        addr.sin_port = htons ((uint16) port);
+        addr.sin_addr.s_addr = address.isNotEmpty() ? ::inet_addr (address.toRawUTF8())
+                                                    : htonl (INADDR_ANY);
 
-        return bind (handle, (struct sockaddr*) &servTmpAddr, sizeof (struct sockaddr_in)) >= 0;
+        return ::bind (handle, (struct sockaddr*) &addr, sizeof (addr)) >= 0;
     }
 
     static int getBoundPort (const SocketHandle handle) noexcept
     {
-        if (handle <= 0)
-            return -1;
+        if (handle > 0)
+        {
+            struct sockaddr_in addr;
+            socklen_t len = sizeof (addr);
 
-        struct sockaddr_in sin_addr;
-        socklen_t len = sizeof (sin_addr);
-
-        if (getsockname (handle, (struct sockaddr*) &sin_addr, &len) == 0)
-            return ntohs (sin_addr.sin_port);
+            if (getsockname (handle, (struct sockaddr*) &addr, &len) == 0)
+                return ntohs (addr.sin_port);
+        }
 
         return -1;
     }
@@ -332,45 +333,58 @@ namespace SocketHelpers
                                const int portNumber,
                                const int timeOutMillisecs) noexcept
     {
-        struct addrinfo* info = getAddressInfo (false, hostName, portNumber);
+        bool success = false;
 
-        if (info == nullptr)
-            return false;
-
-        if (handle < 0)
-            handle = (int) socket (info->ai_family, info->ai_socktype, 0);
-
-        if (handle < 0)
+        if (struct addrinfo* info = getAddressInfo (false, hostName, portNumber))
         {
-            freeaddrinfo (info);
-            return false;
-        }
-
-        setSocketBlockingState (handle, false);
-        const int result = ::connect (handle, info->ai_addr, (socklen_t) info->ai_addrlen);
-        freeaddrinfo (info);
-
-        bool retval = (result >= 0);
-
-        if (result < 0)
-        {
-           #if JUCE_WINDOWS
-            if (result == SOCKET_ERROR && WSAGetLastError() == WSAEWOULDBLOCK)
-           #else
-            if (errno == EINPROGRESS)
-           #endif
+            for (struct addrinfo* i = info; i != nullptr; i = i->ai_next)
             {
-                if (waitForReadiness (handle, readLock, false, timeOutMillisecs) == 1)
-                    retval = true;
+                const SocketHandle newHandle = socket (i->ai_family, i->ai_socktype, 0);
+
+                if (newHandle >= 0)
+                {
+                    setSocketBlockingState (newHandle, false);
+                    const int result = ::connect (newHandle, i->ai_addr, (socklen_t) i->ai_addrlen);
+                    success = (result >= 0);
+
+                    if (! success)
+                    {
+                       #if JUCE_WINDOWS
+                        if (result == SOCKET_ERROR && WSAGetLastError() == WSAEWOULDBLOCK)
+                       #else
+                        if (errno == EINPROGRESS)
+                       #endif
+                        {
+                            const volatile int cvHandle = (int) newHandle;
+                            if (waitForReadiness (cvHandle, readLock, false, timeOutMillisecs) == 1)
+                                success = true;
+                        }
+                    }
+
+                    if (success)
+                    {
+                        handle = (int) newHandle;
+                        break;
+                    }
+
+                   #if JUCE_WINDOWS
+                    closesocket (newHandle);
+                   #else
+                    ::close (newHandle);
+                   #endif
+                }
+            }
+
+            freeaddrinfo (info);
+
+            if (success)
+            {
+                setSocketBlockingState (handle, true);
+                resetSocketOptions (handle, false, false);
             }
         }
 
-        setSocketBlockingState (handle, true);
-
-        if (retval)
-            resetSocketOptions (handle, false, false);
-
-        return retval;
+        return success;
     }
 
     static void makeReusable (int handle) noexcept
@@ -384,11 +398,11 @@ namespace SocketHelpers
         struct ip_mreq mreq;
 
         zerostruct (mreq);
-        mreq.imr_multiaddr.s_addr = inet_addr (multicastIPAddress.toUTF8());
+        mreq.imr_multiaddr.s_addr = inet_addr (multicastIPAddress.toRawUTF8());
         mreq.imr_interface.s_addr = INADDR_ANY;
 
         if (interfaceIPAddress.isNotEmpty())
-            mreq.imr_interface.s_addr = inet_addr (interfaceIPAddress.toUTF8());
+            mreq.imr_interface.s_addr = inet_addr (interfaceIPAddress.toRawUTF8());
 
         return setsockopt (handle, IPPROTO_IP,
                            join ? IP_ADD_MEMBERSHIP
@@ -414,6 +428,8 @@ StreamingSocket::StreamingSocket (const String& host, int portNum, int h)
       connected (true),
       isListener (false)
 {
+    jassert (SocketHelpers::isValidPortNumber (portNum));
+
     SocketHelpers::initSockets();
     SocketHelpers::resetSocketOptions (h, false, false);
 }
@@ -455,6 +471,8 @@ bool StreamingSocket::bindToPort (const int port)
 
 bool StreamingSocket::bindToPort (const int port, const String& addr)
 {
+    jassert (SocketHelpers::isValidPortNumber (port));
+
     return SocketHelpers::bindSocket (handle, port, addr);
 }
 
@@ -467,6 +485,8 @@ bool StreamingSocket::connect (const String& remoteHostName,
                                const int remotePortNumber,
                                const int timeOutMillisecs)
 {
+    jassert (SocketHelpers::isValidPortNumber (remotePortNumber));
+
     if (isListener)
     {
         jassertfalse;    // a listener socket can't connect to another one!
@@ -505,6 +525,8 @@ void StreamingSocket::close()
 //==============================================================================
 bool StreamingSocket::createListener (const int newPortNumber, const String& localHostName)
 {
+    jassert (SocketHelpers::isValidPortNumber (newPortNumber));
+
     if (connected)
         close();
 
@@ -512,17 +534,6 @@ bool StreamingSocket::createListener (const int newPortNumber, const String& loc
     portNumber = newPortNumber;
     isListener = true;
 
-    struct sockaddr_in servTmpAddr;
-    zerostruct (servTmpAddr);
-
-    servTmpAddr.sin_family = PF_INET;
-    servTmpAddr.sin_addr.s_addr = htonl (INADDR_ANY);
-
-    if (localHostName.isNotEmpty())
-        servTmpAddr.sin_addr.s_addr = ::inet_addr (localHostName.toUTF8());
-
-    servTmpAddr.sin_port = htons ((uint16) portNumber);
-
     handle = (int) socket (AF_INET, SOCK_STREAM, 0);
 
     if (handle < 0)
@@ -532,15 +543,15 @@ bool StreamingSocket::createListener (const int newPortNumber, const String& loc
     SocketHelpers::makeReusable (handle);
    #endif
 
-    if (bind (handle, (struct sockaddr*) &servTmpAddr, sizeof (struct sockaddr_in)) < 0
-         || listen (handle, SOMAXCONN) < 0)
+    if (SocketHelpers::bindSocket (handle, portNumber, localHostName)
+         && listen (handle, SOMAXCONN) >= 0)
     {
-        close();
-        return false;
+        connected = true;
+        return true;
     }
 
-    connected = true;
-    return true;
+    close();
+    return false;
 }
 
 StreamingSocket* StreamingSocket::waitForNextConnection() const
@@ -614,14 +625,12 @@ bool DatagramSocket::bindToPort (const int port)
 
 bool DatagramSocket::bindToPort (const int port, const String& addr)
 {
-    if (handle < 0)
-        return false;
+    jassert (SocketHelpers::isValidPortNumber (port));
 
     if (SocketHelpers::bindSocket (handle, port, addr))
     {
         isBound = true;
         lastBindAddress = addr;
-
         return true;
     }
 
@@ -673,6 +682,8 @@ int DatagramSocket::read (void* destBuffer, int maxBytesToRead, bool shouldBlock
 int DatagramSocket::write (const String& remoteHostname, int remotePortNumber,
                            const void* sourceBuffer, int numBytesToWrite)
 {
+    jassert (SocketHelpers::isValidPortNumber (remotePortNumber));
+
     if (handle < 0)
         return -1;
 
