@@ -627,7 +627,9 @@ ValueTree::ValueTree() noexcept
 {
 }
 
+#if JUCE_ALLOW_STATIC_NULL_VARIABLES
 const ValueTree ValueTree::invalid;
+#endif
 
 ValueTree::ValueTree (const Identifier& type)  : object (new ValueTree::SharedObject (type))
 {
@@ -721,20 +723,30 @@ ValueTree ValueTree::getParent() const noexcept
 ValueTree ValueTree::getSibling (const int delta) const noexcept
 {
     if (object == nullptr || object->parent == nullptr)
-        return invalid;
+        return ValueTree();
 
     const int index = object->parent->indexOf (*this) + delta;
     return ValueTree (object->parent->children.getObjectPointer (index));
 }
 
+static const var& getNullVarRef() noexcept
+{
+   #if JUCE_ALLOW_STATIC_NULL_VARIABLES
+    return var::null;
+   #else
+    static var nullVar;
+    return nullVar;
+   #endif
+}
+
 const var& ValueTree::operator[] (const Identifier& name) const noexcept
 {
-    return object == nullptr ? var::null : object->properties[name];
+    return object == nullptr ? getNullVarRef() : object->properties[name];
 }
 
 const var& ValueTree::getProperty (const Identifier& name) const noexcept
 {
-    return object == nullptr ? var::null : object->properties[name];
+    return object == nullptr ? getNullVarRef() : object->properties[name];
 }
 
 var ValueTree::getProperty (const Identifier& name, const var& defaultReturnValue) const
@@ -858,6 +870,29 @@ ValueTree ValueTree::getChild (int index) const
                                         : static_cast<SharedObject*> (nullptr));
 }
 
+ValueTree::Iterator::Iterator (const ValueTree& v, bool isEnd) noexcept
+   : internal (v.object != nullptr ? (isEnd ? v.object->children.end() : v.object->children.begin()) : nullptr)
+{}
+
+ValueTree::Iterator& ValueTree::Iterator::operator++() noexcept
+{
+    internal = static_cast<SharedObject**> (internal) + 1;
+    return *this;
+}
+
+bool ValueTree::Iterator::operator!= (const Iterator& other) const noexcept
+{
+    return internal != other.internal;
+}
+
+ValueTree ValueTree::Iterator::operator*() const
+{
+    return ValueTree (*static_cast<SharedObject**> (internal));
+}
+
+ValueTree::Iterator ValueTree::begin() const noexcept   { return Iterator (*this, false); }
+ValueTree::Iterator ValueTree::end() const noexcept     { return Iterator (*this, true); }
+
 ValueTree ValueTree::getChildWithName (const Identifier& type) const
 {
     return object != nullptr ? object->getChildWithName (type) : ValueTree();
