@@ -25,7 +25,7 @@
 namespace
 {
     template <typename Type>
-    Rectangle<Type> coordsToRectangle (Type x, Type y, Type w, Type h)
+    Rectangle<Type> coordsToRectangle (Type x, Type y, Type w, Type h) noexcept
     {
        #if JUCE_DEBUG
         const int maxVal = 0x3fffffff;
@@ -77,7 +77,7 @@ bool Graphics::isVectorDevice() const
     return context.isVectorDevice();
 }
 
-bool Graphics::reduceClipRegion (const Rectangle<int>& area)
+bool Graphics::reduceClipRegion (Rectangle<int> area)
 {
     saveStateIfPending();
     return context.clipToRectangle (area);
@@ -85,7 +85,7 @@ bool Graphics::reduceClipRegion (const Rectangle<int>& area)
 
 bool Graphics::reduceClipRegion (const int x, const int y, const int w, const int h)
 {
-    return reduceClipRegion (Rectangle<int> (x, y, w, h));
+    return reduceClipRegion (coordsToRectangle (x, y, w, h));
 }
 
 bool Graphics::reduceClipRegion (const RectangleList<int>& clipRegion)
@@ -108,7 +108,7 @@ bool Graphics::reduceClipRegion (const Image& image, const AffineTransform& tran
     return ! context.isClipEmpty();
 }
 
-void Graphics::excludeClipRegion (const Rectangle<int>& rectangleToExclude)
+void Graphics::excludeClipRegion (Rectangle<int> rectangleToExclude)
 {
     saveStateIfPending();
     context.excludeClipRectangle (rectangleToExclude);
@@ -164,7 +164,7 @@ void Graphics::addTransform (const AffineTransform& transform)
     context.addTransform (transform);
 }
 
-bool Graphics::clipRegionIntersects (const Rectangle<int>& area) const
+bool Graphics::clipRegionIntersects (Rectangle<int> area) const
 {
     return context.clipRegionIntersects (area);
 }
@@ -281,7 +281,7 @@ void Graphics::drawMultiLineText (const String& text, const int startX,
     }
 }
 
-void Graphics::drawText (const String& text, const Rectangle<float>& area,
+void Graphics::drawText (const String& text, Rectangle<float> area,
                          Justification justificationType, bool useEllipsesIfTooBig) const
 {
     if (text.isNotEmpty() && context.clipRegionIntersects (area.getSmallestIntegerContainer()))
@@ -297,19 +297,19 @@ void Graphics::drawText (const String& text, const Rectangle<float>& area,
     }
 }
 
-void Graphics::drawText (const String& text, const Rectangle<int>& area,
+void Graphics::drawText (const String& text, Rectangle<int> area,
                          Justification justificationType, bool useEllipsesIfTooBig) const
 {
     drawText (text, area.toFloat(), justificationType, useEllipsesIfTooBig);
 }
 
-void Graphics::drawText (const String& text, const int x, const int y, const int width, const int height,
+void Graphics::drawText (const String& text, int x, int y, int width, int height,
                          Justification justificationType, const bool useEllipsesIfTooBig) const
 {
-    drawText (text, Rectangle<int> (x, y, width, height), justificationType, useEllipsesIfTooBig);
+    drawText (text, coordsToRectangle (x, y, width, height), justificationType, useEllipsesIfTooBig);
 }
 
-void Graphics::drawFittedText (const String& text, const Rectangle<int>& area,
+void Graphics::drawFittedText (const String& text, Rectangle<int> area,
                                Justification justification,
                                const int maximumNumberOfLines,
                                const float minimumHorizontalScale) const
@@ -328,7 +328,7 @@ void Graphics::drawFittedText (const String& text, const Rectangle<int>& area,
     }
 }
 
-void Graphics::drawFittedText (const String& text, const int x, const int y, const int width, const int height,
+void Graphics::drawFittedText (const String& text, int x, int y, int width, int height,
                                Justification justification,
                                const int maximumNumberOfLines,
                                const float minimumHorizontalScale) const
@@ -338,12 +338,12 @@ void Graphics::drawFittedText (const String& text, const int x, const int y, con
 }
 
 //==============================================================================
-void Graphics::fillRect (const Rectangle<int>& r) const
+void Graphics::fillRect (Rectangle<int> r) const
 {
     context.fillRect (r, false);
 }
 
-void Graphics::fillRect (const Rectangle<float>& r) const
+void Graphics::fillRect (Rectangle<float> r) const
 {
     context.fillRect (r);
 }
@@ -371,7 +371,7 @@ void Graphics::fillRectList (const RectangleList<int>& rects) const
 
 void Graphics::setPixel (int x, int y) const
 {
-    context.fillRect (Rectangle<int> (x, y, 1, 1), false);
+    context.fillRect (coordsToRectangle (x, y, 1, 1), false);
 }
 
 void Graphics::fillAll() const
@@ -394,9 +394,15 @@ void Graphics::fillAll (Colour colourToUse) const
 
 
 //==============================================================================
+void Graphics::fillPath (const Path& path) const
+{
+    if (! (context.isClipEmpty() || path.isEmpty()))
+        context.fillPath (path, AffineTransform());
+}
+
 void Graphics::fillPath (const Path& path, const AffineTransform& transform) const
 {
-    if ((! context.isClipEmpty()) && ! path.isEmpty())
+    if (! (context.isClipEmpty() || path.isEmpty()))
         context.fillPath (path, transform);
 }
 
@@ -420,7 +426,7 @@ void Graphics::drawRect (int x, int y, int width, int height, int lineThickness)
     drawRect (coordsToRectangle (x, y, width, height), lineThickness);
 }
 
-void Graphics::drawRect (const Rectangle<int>& r, int lineThickness) const
+void Graphics::drawRect (Rectangle<int> r, int lineThickness) const
 {
     drawRect (r.toFloat(), (float) lineThickness);
 }
@@ -438,7 +444,7 @@ void Graphics::drawRect (Rectangle<float> r, const float lineThickness) const
 }
 
 //==============================================================================
-void Graphics::fillEllipse (const Rectangle<float>& area) const
+void Graphics::fillEllipse (Rectangle<float> area) const
 {
     Path p;
     p.addEllipse (area);
@@ -447,19 +453,31 @@ void Graphics::fillEllipse (const Rectangle<float>& area) const
 
 void Graphics::fillEllipse (float x, float y, float w, float h) const
 {
-    fillEllipse (Rectangle<float> (x, y, w, h));
+    fillEllipse (coordsToRectangle (x, y, w, h));
 }
 
 void Graphics::drawEllipse (float x, float y, float width, float height, float lineThickness) const
 {
-    Path p;
-    p.addEllipse (x, y, width, height);
-    strokePath (p, PathStrokeType (lineThickness));
+    drawEllipse (coordsToRectangle (x, y, width, height), lineThickness);
 }
 
-void Graphics::drawEllipse (const Rectangle<float>& area, float lineThickness) const
+void Graphics::drawEllipse (Rectangle<float> area, float lineThickness) const
 {
-    drawEllipse (area.getX(), area.getY(), area.getWidth(), area.getHeight(), lineThickness);
+    Path p;
+
+    if (area.getWidth() == area.getHeight())
+    {
+        // For a circle, we can avoid having to generate a stroke
+        p.addEllipse (area.expanded (lineThickness * 0.5f));
+        p.addEllipse (area.reduced  (lineThickness * 0.5f));
+        p.setUsingNonZeroWinding (false);
+        fillPath (p);
+    }
+    else
+    {
+        p.addEllipse (area);
+        strokePath (p, PathStrokeType (lineThickness));
+    }
 }
 
 void Graphics::fillRoundedRectangle (float x, float y, float width, float height, float cornerSize) const
@@ -467,7 +485,7 @@ void Graphics::fillRoundedRectangle (float x, float y, float width, float height
     fillRoundedRectangle (coordsToRectangle (x, y, width, height), cornerSize);
 }
 
-void Graphics::fillRoundedRectangle (const Rectangle<float>& r, const float cornerSize) const
+void Graphics::fillRoundedRectangle (Rectangle<float> r, const float cornerSize) const
 {
     Path p;
     p.addRoundedRectangle (r, cornerSize);
@@ -480,7 +498,7 @@ void Graphics::drawRoundedRectangle (float x, float y, float width, float height
     drawRoundedRectangle (coordsToRectangle (x, y, width, height), cornerSize, lineThickness);
 }
 
-void Graphics::drawRoundedRectangle (const Rectangle<float>& r, float cornerSize, float lineThickness) const
+void Graphics::drawRoundedRectangle (Rectangle<float> r, float cornerSize, float lineThickness) const
 {
     Path p;
     p.addRoundedRectangle (r, cornerSize);
@@ -494,7 +512,7 @@ void Graphics::drawArrow (const Line<float>& line, float lineThickness, float ar
     fillPath (p);
 }
 
-void Graphics::fillCheckerBoard (const Rectangle<int>& area,
+void Graphics::fillCheckerBoard (Rectangle<int> area,
                                  const int checkWidth, const int checkHeight,
                                  Colour colour1, Colour colour2) const
 {
@@ -624,18 +642,22 @@ void Graphics::drawImageAt (const Image& imageToDraw, int x, int y, bool fillAlp
                           fillAlphaChannel);
 }
 
-void Graphics::drawImageWithin (const Image& imageToDraw,
-                                int dx, int dy, int dw, int dh,
-                                RectanglePlacement placementWithinTarget,
-                                const bool fillAlphaChannelWithCurrentBrush) const
+void Graphics::drawImage (const Image& imageToDraw, Rectangle<float> targetArea,
+                          RectanglePlacement placementWithinTarget, bool fillAlphaChannelWithCurrentBrush) const
 {
     if (imageToDraw.isValid())
         drawImageTransformed (imageToDraw,
-                              placementWithinTarget.getTransformToFit (imageToDraw.getBounds().toFloat(),
-                                                                       coordsToRectangle (dx, dy, dw, dh).toFloat()),
+                              placementWithinTarget.getTransformToFit (imageToDraw.getBounds().toFloat(), targetArea),
                               fillAlphaChannelWithCurrentBrush);
 }
 
+void Graphics::drawImageWithin (const Image& imageToDraw, int dx, int dy, int dw, int dh,
+                                RectanglePlacement placementWithinTarget, bool fillAlphaChannelWithCurrentBrush) const
+{
+    drawImage (imageToDraw, coordsToRectangle (dx, dy, dw, dh).toFloat(),
+               placementWithinTarget, fillAlphaChannelWithCurrentBrush);
+}
+
 void Graphics::drawImage (const Image& imageToDraw,
                           int dx, int dy, int dw, int dh,
                           int sx, int sy, int sw, int sh,
