@@ -83,12 +83,14 @@ public:
     bool operator== (const Path&) const noexcept;
     bool operator!= (const Path&) const noexcept;
 
+    static const float defaultToleranceForTesting;
+    static const float defaultToleranceForMeasurement;
+
     //==============================================================================
     /** Returns true if the path doesn't contain any lines or curves. */
     bool isEmpty() const noexcept;
 
-    /** Returns the smallest rectangle that contains all points within the path.
-    */
+    /** Returns the smallest rectangle that contains all points within the path. */
     Rectangle<float> getBounds() const noexcept;
 
     /** Returns the smallest rectangle that contains all points within the path
@@ -98,7 +100,7 @@ public:
 
     /** Checks whether a point lies within the path.
 
-        This is only relevent for closed paths (see closeSubPath()), and
+        This is only relevant for closed paths (see closeSubPath()), and
         may produce false results if used on a path which has open sub-paths.
 
         The path's winding rule is taken into account by this method.
@@ -110,11 +112,11 @@ public:
         @see closeSubPath, setUsingNonZeroWinding
     */
     bool contains (float x, float y,
-                   float tolerance = 1.0f) const;
+                   float tolerance = defaultToleranceForTesting) const;
 
     /** Checks whether a point lies within the path.
 
-        This is only relevent for closed paths (see closeSubPath()), and
+        This is only relevant for closed paths (see closeSubPath()), and
         may produce false results if used on a path which has open sub-paths.
 
         The path's winding rule is taken into account by this method.
@@ -126,7 +128,7 @@ public:
         @see closeSubPath, setUsingNonZeroWinding
     */
     bool contains (const Point<float> point,
-                   float tolerance = 1.0f) const;
+                   float tolerance = defaultToleranceForTesting) const;
 
     /** Checks whether a line crosses the path.
 
@@ -138,8 +140,8 @@ public:
         so this method could return a false positive when your point is up to this distance
         outside the path's boundary.
     */
-    bool intersectsLine (const Line<float>& line,
-                         float tolerance = 1.0f);
+    bool intersectsLine (Line<float> line,
+                         float tolerance = defaultToleranceForTesting);
 
     /** Cuts off parts of a line to keep the parts that are either inside or
         outside this path.
@@ -153,12 +155,13 @@ public:
                                         that will be kept; if false its the section inside
                                         the path
     */
-    Line<float> getClippedLine (const Line<float>& line, bool keepSectionOutsidePath) const;
+    Line<float> getClippedLine (Line<float> line, bool keepSectionOutsidePath) const;
 
     /** Returns the length of the path.
         @see getPointAlongPath
     */
-    float getLength (const AffineTransform& transform = AffineTransform::identity) const;
+    float getLength (const AffineTransform& transform = AffineTransform(),
+                     float tolerance = defaultToleranceForMeasurement) const;
 
     /** Returns a point that is the specified distance along the path.
         If the distance is greater than the total length of the path, this will return the
@@ -166,15 +169,17 @@ public:
         @see getLength
     */
     Point<float> getPointAlongPath (float distanceFromStart,
-                                    const AffineTransform& transform = AffineTransform::identity) const;
+                                    const AffineTransform& transform = AffineTransform(),
+                                    float tolerance = defaultToleranceForMeasurement) const;
 
     /** Finds the point along the path which is nearest to a given position.
         This sets pointOnPath to the nearest point, and returns the distance of this point from the start
         of the path.
     */
-    float getNearestPoint (const Point<float> targetPoint,
+    float getNearestPoint (Point<float> targetPoint,
                            Point<float>& pointOnPath,
-                           const AffineTransform& transform = AffineTransform::identity) const;
+                           const AffineTransform& transform = AffineTransform(),
+                           float tolerance = defaultToleranceForMeasurement) const;
 
     //==============================================================================
     /** Removes all lines and curves, resetting the path completely. */
