@@ -22,7 +22,6 @@
   ==============================================================================
 */
 
-//==================================================================================================
 #if JUCE_USE_DIRECTWRITE
 namespace DirectWriteTypeLayout
 {
@@ -41,7 +40,7 @@ namespace DirectWriteTypeLayout
         JUCE_COMRESULT QueryInterface (REFIID refId, void** result) override
         {
             if (refId == __uuidof (IDWritePixelSnapping))
-                return castToType <IDWritePixelSnapping> (result);
+                return castToType<IDWritePixelSnapping> (result);
 
             return ComBaseClassHelper<IDWriteTextRenderer>::QueryInterface (refId, result);
         }
@@ -54,12 +53,9 @@ namespace DirectWriteTypeLayout
 
         JUCE_COMRESULT GetCurrentTransform (void*, DWRITE_MATRIX* matrix) override
         {
-            matrix->m11 = 1.0f;
-            matrix->m12 = 0.0f;
-            matrix->m21 = 0.0f;
-            matrix->m22 = 1.0f;
-            matrix->dx = 0.0f;
-            matrix->dy = 0.0f;
+            matrix->m11 = 1.0f; matrix->m12 = 0.0f;
+            matrix->m21 = 0.0f; matrix->m22 = 1.0f;
+            matrix->dx  = 0.0f; matrix->dy  = 0.0f;
             return S_OK;
         }
 
@@ -131,19 +127,22 @@ namespace DirectWriteTypeLayout
             const Point<float> lineOrigin (layout->getLine (currentLine).lineOrigin);
             float x = baselineOriginX - lineOrigin.x;
 
+            const float extraKerning = glyphRunLayout->font.getExtraKerningFactor()
+                                          * glyphRunLayout->font.getHeight();
+
             for (UINT32 i = 0; i < glyphRun->glyphCount; ++i)
             {
                 const float advance = glyphRun->glyphAdvances[i];
 
                 if ((glyphRun->bidiLevel & 1) != 0)
-                    x -= advance;  // RTL text
+                    x -= advance + extraKerning;  // RTL text
 
                 glyphRunLayout->glyphs.add (TextLayout::Glyph (glyphRun->glyphIndices[i],
                                                                Point<float> (x, baselineOriginY - lineOrigin.y),
                                                                advance));
 
                 if ((glyphRun->bidiLevel & 1) == 0)
-                    x += advance;  // LTR text
+                    x += advance + extraKerning;  // LTR text
             }
 
             return S_OK;
@@ -172,10 +171,13 @@ namespace DirectWriteTypeLayout
         Font getFontForRun (const DWRITE_GLYPH_RUN& glyphRun, float fontHeight)
         {
             for (int i = 0; i < attributedString.getNumAttributes(); ++i)
-                if (const Font* font = attributedString.getAttribute(i)->getFont())
-                    if (WindowsDirectWriteTypeface* wt = dynamic_cast<WindowsDirectWriteTypeface*> (font->getTypeface()))
-                        if (wt->getIDWriteFontFace() == glyphRun.fontFace)
-                            return font->withHeight (fontHeight);
+            {
+                const Font& font = attributedString.getAttribute(i).font;
+
+                if (WindowsDirectWriteTypeface* wt = dynamic_cast<WindowsDirectWriteTypeface*> (font.getTypeface()))
+                    if (wt->getIDWriteFontFace() == glyphRun.fontFace)
+                        return font.withHeight (fontHeight);
+            }
 
             ComSmartPtr<IDWriteFont> dwFont;
             HRESULT hr = fontCollection.GetFontFromFontFace (glyphRun.fontFace, dwFont.resetAndGetPointerAddress());
@@ -190,7 +192,7 @@ namespace DirectWriteTypeLayout
         JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (CustomDirectWriteTextRenderer)
     };
 
-    //==================================================================================================
+    //==============================================================================
     static float getFontHeightToEmSizeFactor (IDWriteFont& dwFont)
     {
         ComSmartPtr<IDWriteFontFace> dwFontFace;
@@ -253,9 +255,8 @@ namespace DirectWriteTypeLayout
         range.startPosition = attr.range.getStart();
         range.length = jmin (attr.range.getLength(), textLen - attr.range.getStart());
 
-        if (const Font* const font = attr.getFont())
         {
-            const String familyName (FontStyleHelpers::getConcreteFamilyName (*font));
+            const String familyName (FontStyleHelpers::getConcreteFamilyName (attr.font));
 
             BOOL fontFound = false;
             uint32 fontIndex;
@@ -275,7 +276,7 @@ namespace DirectWriteTypeLayout
             {
                 hr = fontFamily->GetFont (i, dwFont.resetAndGetPointerAddress());
 
-                if (font->getTypefaceStyle() == getFontFaceName (dwFont))
+                if (attr.font.getTypefaceStyle() == getFontFaceName (dwFont))
                     break;
             }
 
@@ -285,16 +286,16 @@ namespace DirectWriteTypeLayout
             textLayout.SetFontStyle (dwFont->GetStyle(), range);
 
             const float fontHeightToEmSizeFactor = getFontHeightToEmSizeFactor (*dwFont);
-            textLayout.SetFontSize (font->getHeight() * fontHeightToEmSizeFactor, range);
+            textLayout.SetFontSize (attr.font.getHeight() * fontHeightToEmSizeFactor, range);
         }
 
-        if (const Colour* const colour = attr.getColour())
         {
+            const Colour col (attr.colour);
             ComSmartPtr<ID2D1SolidColorBrush> d2dBrush;
-            renderTarget.CreateSolidColorBrush (D2D1::ColorF (colour->getFloatRed(),
-                                                              colour->getFloatGreen(),
-                                                              colour->getFloatBlue(),
-                                                              colour->getFloatAlpha()),
+            renderTarget.CreateSolidColorBrush (D2D1::ColorF (col.getFloatRed(),
+                                                              col.getFloatGreen(),
+                                                              col.getFloatBlue(),
+                                                              col.getFloatAlpha()),
                                                 d2dBrush.resetAndGetPointerAddress());
 
             // We need to call SetDrawingEffect with a legimate brush to get DirectWrite to break text based on colours
@@ -353,7 +354,7 @@ namespace DirectWriteTypeLayout
         const int numAttributes = text.getNumAttributes();
 
         for (int i = 0; i < numAttributes; ++i)
-            addAttributedRange (*text.getAttribute (i), *textLayout, textLen, renderTarget, fontCollection);
+            addAttributedRange (text.getAttribute (i), *textLayout, textLen, renderTarget, fontCollection);
 
         return true;
     }
@@ -419,9 +420,8 @@ static bool canAllTypefacesBeUsedInLayout (const AttributedString& text)
     const int numCharacterAttributes = text.getNumAttributes();
 
     for (int i = 0; i < numCharacterAttributes; ++i)
-        if (const Font* const font = text.getAttribute (i)->getFont())
-            if (dynamic_cast<WindowsDirectWriteTypeface*> (font->getTypeface()) == nullptr)
-                return false;
+        if (dynamic_cast<WindowsDirectWriteTypeface*> (text.getAttribute(i).font.getTypeface()) == nullptr)
+            return false;
 
     return true;
 }
