@@ -175,8 +175,15 @@ void Button::setToggleState (const bool shouldBeOn, const NotificationType notif
                 return;
         }
 
-        if (getToggleState() != shouldBeOn)  // this test means that if the value is void rather than explicitly set to
-            isOn = shouldBeOn;               // false, it won't be changed unless the required value is true.
+        // This test is done so that if the value is void rather than explicitly set to
+        // false, the value won't be changed unless the required value is true.
+        if (getToggleState() != shouldBeOn)
+        {
+            isOn = shouldBeOn;
+
+            if (deletionWatcher == nullptr)
+                return;
+        }
 
         lastToggleState = shouldBeOn;
         repaint();
@@ -456,7 +463,7 @@ void Button::mouseUp (const MouseEvent& e)
 {
     const bool wasDown = isDown();
     const bool wasOver = isOver();
-    updateState (isMouseOver(), false);
+    updateState (isMouseOrTouchOver (e), false);
 
     if (wasDown && wasOver && ! triggerOnMouseDown)
     {
@@ -467,15 +474,23 @@ void Button::mouseUp (const MouseEvent& e)
     }
 }
 
-void Button::mouseDrag (const MouseEvent&)
+void Button::mouseDrag (const MouseEvent& e)
 {
     const ButtonState oldState = buttonState;
-    updateState (isMouseOver(), true);
+    updateState (isMouseOrTouchOver (e), true);
 
     if (autoRepeatDelay >= 0 && buttonState != oldState && isDown())
         callbackHelper->startTimer (autoRepeatSpeed);
 }
 
+bool Button::isMouseOrTouchOver (const MouseEvent& e)
+{
+    if (e.source.isTouch())
+        return getLocalBounds().toFloat().contains (e.position);
+
+    return isMouseOver();
+}
+
 void Button::focusGained (FocusChangeType)
 {
     updateState();
