@@ -146,7 +146,7 @@ public:
     void sendMouseUp (Component& comp, Point<float> screenPos, Time time, const ModifierKeys oldMods)
     {
         JUCE_MOUSE_EVENT_DBG ("up")
-        comp.internalMouseUp (MouseInputSource (this), screenPosToLocalPos (comp, screenPos), time, oldMods);
+        comp.internalMouseUp (MouseInputSource (this), screenPosToLocalPos (comp, screenPos), time, oldMods, pressure);
     }
 
     void sendMouseWheel (Component& comp, Point<float> screenPos, Time time, const MouseWheelDetails& wheel)
@@ -330,7 +330,7 @@ public:
         setScreenPos (screenPos, time, false);
         triggerFakeMove();
 
-        return isDragging() ? nullptr : getComponentUnderMouse();
+        return getComponentUnderMouse();
     }
 
     void handleWheel (ComponentPeer& peer, Point<float> positionWithinPeer,
