@@ -138,22 +138,35 @@ public:
     SliderStyle getSliderStyle() const noexcept;
 
     //==============================================================================
-    /** Changes the properties of a rotary slider.
+    struct RotaryParameters
+    {
+        /** The angle (in radians, clockwise from the top) at which
+            the slider's minimum value is represented. */
+        float startAngleRadians;
 
-        @param startAngleRadians    the angle (in radians, clockwise from the top) at which
-                                    the slider's minimum value is represented
-        @param endAngleRadians      the angle (in radians, clockwise from the top) at which
-                                    the slider's maximum value is represented. This must be
-                                    greater than startAngleRadians
-        @param stopAtEnd            determines what happens when a circular drag action rotates beyond
-                                    the minimum or maximum angle. If true, the value will stop changing
-                                    until the mouse moves back the way it came; if false, the value
-                                    will snap back to the value nearest to the mouse. Note that this has
-                                    no effect if the drag mode is vertical or horizontal.
-    */
+        /** The angle (in radians, clockwise from the top) at which
+            the slider's maximum value is represented. This must be
+            greater than startAngleRadians. */
+        float endAngleRadians;
+
+        /** Determines what happens when a circular drag action rotates beyond
+            the minimum or maximum angle. If true, the value will stop changing
+            until the mouse moves back the way it came; if false, the value
+            will snap back to the value nearest to the mouse. Note that this has
+            no effect if the drag mode is vertical or horizontal.*/
+        bool stopAtEnd;
+    };
+
+    /** Changes the properties of a rotary slider. */
+    void setRotaryParameters (RotaryParameters newParameters) noexcept;
+
+    /** Changes the properties of a rotary slider. */
     void setRotaryParameters (float startAngleRadians,
                               float endAngleRadians,
-                              bool stopAtEnd);
+                              bool stopAtEnd) noexcept;
+
+    /** Changes the properties of a rotary slider. */
+    RotaryParameters getRotaryParameters() const noexcept;
 
     /** Sets the distance the mouse has to move to drag the slider across
         the full extent of its range.
@@ -231,28 +244,37 @@ public:
         slider's length; if the factor is > 1.0, the upper end of the range
         will be expanded instead. A factor of 1.0 doesn't skew it at all.
 
+        If symmetricSkew is true, the skew factor applies from the middle of the slider
+        to each of its ends.
+
         To set the skew position by using a mid-point, use the setSkewFactorFromMidPoint()
         method instead.
 
-        @see getSkewFactor, setSkewFactorFromMidPoint
+        @see getSkewFactor, setSkewFactorFromMidPoint, isSymmetricSkew
     */
-    void setSkewFactor (double factor);
+    void setSkewFactor (double factor, bool symmetricSkew = false);
 
     /** Sets up a skew factor to alter the way values are distributed.
 
         This allows you to specify the slider value that should appear in the
         centre of the slider's visible range.
 
-        @see setSkewFactor, getSkewFactor
+        @see setSkewFactor, getSkewFactor, isSymmetricSkew
      */
     void setSkewFactorFromMidPoint (double sliderValueToShowAtMidPoint);
 
     /** Returns the current skew factor.
         See setSkewFactor for more info.
-        @see setSkewFactor, setSkewFactorFromMidPoint
+        @see setSkewFactor, setSkewFactorFromMidPoint, isSymmetricSkew
     */
     double getSkewFactor() const noexcept;
 
+    /** Returns the whether the skew is symmetric from the midpoint to both sides.
+        See setSkewFactor for more info.
+        @see getSkewFactor, setSkewFactor, setSkewFactorFromMidPoint
+     */
+    bool isSymmetricSkew() const noexcept;
+
     //==============================================================================
     /** Used by setIncDecButtonsMode().
     */
