@@ -95,7 +95,7 @@ String TableHeaderComponent::getColumnName (const int columnId) const
     if (const ColumnInfo* const ci = getInfoForId (columnId))
         return ci->name;
 
-    return String::empty;
+    return String();
 }
 
 void TableHeaderComponent::setColumnName (const int columnId, const String& newName)
@@ -590,7 +590,8 @@ void TableHeaderComponent::mouseDrag (const MouseEvent& e)
 {
     if (columnIdBeingResized == 0
          && columnIdBeingDragged == 0
-         && ! (e.mouseWasClicked() || e.mods.isPopupMenu()))
+         && e.mouseWasDraggedSinceMouseDown()
+         && ! e.mods.isPopupMenu())
     {
         dragOverlayComp = nullptr;
 
@@ -599,6 +600,7 @@ void TableHeaderComponent::mouseDrag (const MouseEvent& e)
         if (columnIdBeingResized != 0)
         {
             const ColumnInfo* const ci = getInfoForId (columnIdBeingResized);
+            jassert (ci != nullptr);
             initialColumnWidth = ci->width;
         }
         else
@@ -767,7 +769,7 @@ void TableHeaderComponent::mouseUp (const MouseEvent& e)
 
     updateColumnUnderMouse (e);
 
-    if (columnIdUnderMouse != 0 && e.mouseWasClicked() && ! e.mods.isPopupMenu())
+    if (columnIdUnderMouse != 0 && ! (e.mouseWasDraggedSinceMouseDown() || e.mods.isPopupMenu()))
         columnClicked (columnIdUnderMouse, e.mods);
 
     dragOverlayComp = nullptr;
