@@ -915,6 +915,7 @@ TextEditor::TextEditor (const String& name,
       menuActive (false),
       valueTextNeedsUpdating (false),
       consumeEscAndReturnKeys (true),
+      styleChanged (false),
       leftIndent (4),
       topIndent (4),
       lastTransactionTime (0),
@@ -1080,15 +1081,23 @@ void TextEditor::colourChanged()
 {
     setOpaque (findColour (backgroundColourId).isOpaque());
     repaint();
+    styleChanged = true;
 }
 
 void TextEditor::lookAndFeelChanged()
 {
+    colourChanged();
+
     caret = nullptr;
     recreateCaret();
     repaint();
 }
 
+void TextEditor::parentHierarchyChanged()
+{
+    lookAndFeelChanged();
+}
+
 void TextEditor::enablementChanged()
 {
     recreateCaret();
@@ -1186,7 +1195,7 @@ void TextEditor::setText (const String& newText,
 {
     const int newLength = newText.length();
 
-    if (newLength != getTotalNumChars() || getText() != newText)
+    if (newLength != getTotalNumChars() || getText() != newText || styleChanged)
     {
         textValue = newText;
 
@@ -1212,6 +1221,8 @@ void TextEditor::setText (const String& newText,
         scrollToMakeSureCursorIsVisible();
         undoManager.clearUndoHistory();
 
+        styleChanged = false;
+
         repaint();
     }
 }
