@@ -593,6 +593,8 @@ private:
 
         jassert (alertBox != nullptr); // you have to return one of these!
 
+        alertBox->setAlwaysOnTop (juce_areThereAnyAlwaysOnTopWindows());
+
        #if JUCE_MODAL_LOOPS_PERMITTED
         if (modal)
         {
@@ -601,7 +603,7 @@ private:
         else
        #endif
         {
-            (void) modal; // (to avoid an unused variable warning)
+            ignoreUnused (modal);
 
             alertBox->enterModalState (true, callback, true);
             alertBox.release();
